package eu.jonahbauer.wizard.client.libgdx;

import com.badlogic.gdx.Game;
import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Graphics;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.audio.Music;
import com.badlogic.gdx.graphics.Pixmap;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.utils.I18NBundle;
import eu.jonahbauer.wizard.client.libgdx.screens.MainMenuScreen;

import java.util.Locale;

public class WizardGame extends Game {
	public static final boolean DEBUG = false;
	public static final int WIDTH = 1920;
	public static final int HEIGHT = 1080;

	public SpriteBatch batch;
	public I18NBundle messages;

	private boolean toggle;

	private int oldHeight, oldWidth;

	@Override
	public void create() {
		batch = new SpriteBatch();
		messages = I18NBundle.createBundle(Gdx.files.internal("i18n/messages"), Locale.getDefault());

		// background music
		Music backgroundMusic = Gdx.audio.newMusic(Gdx.files.internal("background.wav"));
		backgroundMusic.setLooping(true);
		backgroundMusic.setVolume(0.07f);
		backgroundMusic.play();

		// set cursor
		Pixmap cursor = new Pixmap(Gdx.files.internal("cursor.png"));
		Gdx.graphics.setCursor(Gdx.graphics.newCursor(cursor, 0, 0));
		cursor.dispose();
		
		this.setScreen(new MainMenuScreen(this));
	}

	@Override
	public void render() {
		super.render();

		// alt + enter shortcut for fullscreen
		var enter = Gdx.input.isKeyPressed(Input.Keys.ENTER);
		var alt = (Gdx.input.isKeyPressed(Input.Keys.ALT_LEFT) || Gdx.input.isKeyPressed(Input.Keys.ALT_RIGHT));
		var toggle = enter && alt;

		if (toggle && !this.toggle) {
			this.toggle = true;
			var fullscreen = Gdx.graphics.isFullscreen();
			Graphics.DisplayMode displayMode = Gdx.graphics.getDisplayMode();
			if (fullscreen) {
				Gdx.graphics.setWindowedMode(oldWidth, oldHeight);
			} else {
				oldWidth = Gdx.graphics.getWidth();
				oldHeight = Gdx.graphics.getHeight();
				Gdx.graphics.setFullscreenMode(displayMode);
			}
		} else if (!toggle) {
			this.toggle = false;
		}
	}

	@Override
	public void dispose () {
		batch.dispose();
	}
}
