package eu.jonahbauer.wizard.client.libgdx.screens;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.ui.*;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;
import eu.jonahbauer.wizard.client.libgdx.WizardGame;
import eu.jonahbauer.wizard.client.libgdx.actors.AutoFocusScrollPane;
import eu.jonahbauer.wizard.common.messages.data.SessionData;
import eu.jonahbauer.wizard.common.model.Configuration;

import java.util.UUID;
import java.util.stream.IntStream;
import java.util.stream.StreamSupport;

public class LobbyScreen extends MenuScreen {

    private TextButton buttonBack;
    private TextButton buttonJoin;
    private TextButton buttonCreate;

    private TextField playerName;
    private Label labelSessionName;
    private Label labelSessionPlayerCount;
    private Label labelSessionConfiguration;

    private List<SessionData> sessions;

    private final ChangeListener listener = new ChangeListener() {
        @Override
        public void changed(ChangeEvent event, Actor actor) {
            if (actor == buttonBack) {
                game.setScreen(new ConnectScreen(game, data));
                sfxClick();
            } else if (actor == buttonJoin) {
                game.setScreen(new WaitingScreen(game, data));
                sfxClick();
            } else if (actor == buttonCreate) {
                game.setScreen(new CreateGameScreen(game, data));
                sfxClick();
            }
        }
    };

    public LobbyScreen(WizardGame game, Data data) {
        super(game, data);
    }

    @Override
    public void show() {
        super.show();

        buttonBack = new TextButton(game.messages.get("menu.lobby.back"), data.skin);
        buttonJoin = new TextButton(game.messages.get("menu.lobby.join"), data.skin);
        buttonJoin.setDisabled(true);
        buttonCreate = new TextButton(game.messages.get("menu.lobby.create"), data.skin);

        sessions = new List<>(data.skin) {
            @Override
            public String toString(SessionData session) {
                return session.getName();
            }
        };
        sessions.addListener(new ChangeListener() {
            @Override
            public void changed(ChangeEvent event, Actor actor) {
                updateData(sessions.getSelected());
            }
        });

        var listContainer = new AutoFocusScrollPane(sessions, data.skin);
        listContainer.layout();

        var content = new HorizontalGroup().grow().space(20);
        content.setPosition(0.25f * WizardGame.WIDTH, 0.3f * WizardGame.HEIGHT);
        content.setSize(0.5f * WizardGame.WIDTH, 400);
        content.addActor(new Container<>(listContainer).width(0.2f * WizardGame.WIDTH).height(400));
        content.addActor(createInfoTable());
        content.layout();

        var buttons = new HorizontalGroup();
        buttons.setPosition(WizardGame.WIDTH * 0.2f, BUTTON_BAR_Y);
        buttons.setSize(WizardGame.WIDTH * 0.60f, 125);
        buttons.addActor(buttonBack);
        buttons.addActor(buttonCreate);
        buttons.addActor(buttonJoin);
        buttons.space(Math.max(0, (float) (buttons.getWidth() - StreamSupport.stream(buttons.getChildren().spliterator(), false).mapToDouble(Actor::getWidth).sum()) / (buttons.getChildren().size - 1)));

        Gdx.input.setInputProcessor(data.stage);
        data.stage.addActor(content);
        data.stage.addActor(buttons);

        buttonBack.addListener(listener);
        buttonJoin.addListener(listener);
        buttonCreate.addListener(listener);

        addSessions(
                IntStream.range(1, 16)
                         .mapToObj(i -> new SessionData(
                                 UUID.randomUUID(),
                                 "Session " + i,
                                 i % 5,
                                 Configuration.values()[i % Configuration.values().length]
                         ))
                         .toArray(SessionData[]::new)
        );
    }

    @Override
    protected void renderInternal(float delta) {}

    public void addSessions(SessionData...sessions) {
        for (SessionData session : sessions) {
            this.sessions.getItems().add(session);
        }
        this.sessions.layout();
        ((ScrollPane)this.sessions.getParent()).layout();
    }

    private void updateData(SessionData data) {
        System.out.println(data);
        labelSessionName.setText(data.getName());
        labelSessionPlayerCount.setText(Integer.toString(data.getPlayerCount()));
        labelSessionConfiguration.setText(data.getConfiguration().toString());
        buttonJoin.setDisabled(data == null);
    }

    private Table createInfoTable() {
        float infoTableWidth = 0.3f * WizardGame.WIDTH - 20;

        playerName = new TextField("", data.skin);
        labelSessionName = new Label("", data.skin, "textfield");
        labelSessionConfiguration = new Label("", data.skin, "textfield");
        labelSessionPlayerCount = new Label("", data.skin, "textfield");

        labelSessionName.setEllipsis(true);
        labelSessionConfiguration.setEllipsis(true);
        labelSessionPlayerCount.setEllipsis(true);

        var infoTable = new Table().center().left();
        infoTable.columnDefaults(0).growX().width(infoTableWidth);
        infoTable.setSize(infoTableWidth, 400);

        infoTable.add(new Label(game.messages.get("menu.lobby.player_name.label"), data.skin)).row();
        infoTable.add(playerName).row();
        infoTable.add(new Label(game.messages.get("menu.lobby.session_name.label"), data.skin)).row();
        infoTable.add(labelSessionName).row();
        infoTable.add(new Label(game.messages.get("menu.lobby.session_configuration.label"), data.skin)).row();
        infoTable.add(labelSessionConfiguration).row();
        infoTable.add(new Label(game.messages.get("menu.lobby.session_player_count.label"), data.skin)).row();
        infoTable.add(labelSessionPlayerCount).row();

        return infoTable;
    }
}
