package eu.jonahbauer.wizard.client.libgdx.screens;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.ui.*;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;
import eu.jonahbauer.wizard.client.libgdx.WizardGame;
import eu.jonahbauer.wizard.client.libgdx.actors.AutoFocusScrollPane;
import eu.jonahbauer.wizard.common.messages.data.PlayerData;
import eu.jonahbauer.wizard.common.messages.data.SessionData;
import eu.jonahbauer.wizard.common.model.Configuration;

import java.util.UUID;

public class WaitingScreen extends MenuScreen {

    private TextButton buttonLeave;
    private TextButton buttonReady;

    private SessionData session;
    private UUID myUUID;
    private String myName;
    private boolean myReady;
    private List<PlayerData> players;

    {
        // sample data
        session = new SessionData(UUID.randomUUID(), "Session X", -1, Configuration.DEFAULT);
        myUUID = UUID.randomUUID();
        myName = "Max Mustermann";
    }

    private final ChangeListener listener = new ChangeListener() {
        @Override
        public void changed(ChangeEvent event, Actor actor) {
            if (actor == buttonLeave) {
                game.setScreen(new LobbyScreen(game, data));
                sfxClick();
            } else if (actor == buttonReady) {
                ready(!myReady);
                sfxClick();
            }
        }
    };

    public WaitingScreen(WizardGame game, Data data) {
        super(game, data);
    }

    @Override
    public void show() {
        super.show();

        buttonLeave = new TextButton(game.messages.get("menu.waiting.leave"), data.skin);
        buttonLeave.setPosition(WizardGame.WIDTH * 0.275f, BUTTON_BAR_Y);
        buttonReady = new TextButton(game.messages.get("menu.waiting.ready"), data.skin);
        buttonReady.setPosition(WizardGame.WIDTH * 0.725f - buttonReady.getWidth(), BUTTON_BAR_Y);

        players = new List<>(data.skin) {
            @Override
            public String toString(PlayerData player) {
                return player.getName();
            }
        };

        var listContainer = new AutoFocusScrollPane(players, data.skin);
        listContainer.layout();

        var content = new HorizontalGroup().grow().space(20);
        content.setPosition(0.25f * WizardGame.WIDTH, 0.3f * WizardGame.HEIGHT);
        content.setSize(0.5f * WizardGame.WIDTH, 400);
        content.addActor(new Container<>(listContainer).width(0.2f * WizardGame.WIDTH).height(400));
        content.addActor(createInfoTable());
        content.layout();

        Gdx.input.setInputProcessor(data.stage);
        data.stage.addActor(buttonLeave);
        data.stage.addActor(buttonReady);
        data.stage.addActor(content);

        buttonLeave.addListener(listener);
        buttonReady.addListener(listener);

        // sample data
        for (int i = 1; i <= 4; i++) {
            players.getItems().add(new PlayerData(UUID.randomUUID(), "Player " + i, false));
        }
        players.layout();
    }

    private void ready(boolean ready) {
        this.myReady = ready;
        buttonReady.setText(game.messages.get(ready ? "menu.waiting.not_ready" : "menu.waiting.ready"));
    }

    @Override
    protected void renderInternal(float delta) {}

    private Table createInfoTable() {
        float infoTableWidth = 0.3f * WizardGame.WIDTH - 20;

        var labelSessionName = new Label(session.getName(), data.skin, "textfield");
        var labelSessionUUID = new Label(session.getUuid().toString(), data.skin, "textfield");
        var labelSessionConfiguration = new Label(session.getConfiguration().toString(), data.skin, "textfield");
        var labelPlayerName = new Label(myName, data.skin, "textfield");

        labelSessionName.setEllipsis(true);
        labelSessionUUID.setEllipsis(true);
        labelSessionConfiguration.setEllipsis(true);
        labelPlayerName.setEllipsis(true);

        var infoTable = new Table().center().left();
        infoTable.columnDefaults(0).growX().width(infoTableWidth);
        infoTable.setSize(infoTableWidth, 400);

        infoTable.add(new Label(game.messages.get("menu.waiting.session_name.label"), data.skin)).row();
        infoTable.add(labelSessionName).row();
        infoTable.add(new Label(game.messages.get("menu.waiting.session_uuid.label"), data.skin)).row();
        infoTable.add(labelSessionUUID).row();
        infoTable.add(new Label(game.messages.get("menu.waiting.session_configuration.label"), data.skin)).row();
        infoTable.add(labelSessionConfiguration).row();
        infoTable.add(new Label(game.messages.get("menu.waiting.player_name.label"), data.skin)).row();
        infoTable.add(labelPlayerName).row();

        return infoTable;
    }
}
