package eu.jonahbauer.wizard.server.machine.states;

import eu.jonahbauer.wizard.common.messages.client.ClientMessage;
import eu.jonahbauer.wizard.common.messages.client.CreateSessionMessage;
import eu.jonahbauer.wizard.common.messages.client.JoinSessionMessage;
import eu.jonahbauer.wizard.common.messages.server.NackMessage;
import eu.jonahbauer.wizard.server.Lobby;
import eu.jonahbauer.wizard.server.NackException;
import eu.jonahbauer.wizard.server.machine.ClientState;
import eu.jonahbauer.wizard.server.machine.Player;
import org.springframework.web.socket.CloseStatus;

import java.util.Optional;

public class LobbyState implements ClientState {
    @Override
    public Optional<ClientState> onEnter(Player player) {
        Lobby.getInstance().join(player);
        return Optional.empty();
    }

    @Override
    public Optional<ClientState> onMessage(Player player, ClientMessage message) {
        if (message instanceof CreateSessionMessage create) {
            Lobby.getInstance().leave(player);
            var session = Lobby.getInstance().createSession(create);
            var uuid = session.join(player, create.getPlayerName());
            return Optional.of(new SessionState(session.getUuid(), uuid, create.getPlayerName()));
        } else if (message instanceof JoinSessionMessage join) {
            var session = Lobby.getInstance().getSession(join.getSession());
            if (session == null) {
                throw new NackException(NackMessage.NOT_FOUND, "Session not found.");
            } else {
                var uuid = session.join(player, join.getPlayerName());
                return Optional.of(new SessionState(session.getUuid(), uuid, join.getPlayerName()));
            }
        } else {
            return ClientState.super.onMessage(player, message);
        }
    }

    @Override
    public Optional<ClientState> onClose(Player player, CloseStatus status) {
        return Optional.of(new Closed());
    }

    @Override
    public void onExit(Player player) {
        Lobby.getInstance().leave(player);
    }
}
